﻿using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq.Dynamic.Core;
using System.Reflection;
using System.Text;

namespace System.Linq
{
  public static class LinqExtension
  {

    public static IQueryable Search(this IQueryable items, String sortBy, String sortDirection, Int32 skip, Int32 take, String search, IEnumerable<String> validSearchFields = null)
    {
      var itemType = items.ElementType;

      var propertyNames = itemType.GetProperties(BindingFlags.Public | BindingFlags.Instance).Select(p => p.Name).ToList();

      // Apply filtering to all visible column names
      if (!String.IsNullOrEmpty(search))
      {
        // Use only valid fieldnames
        if (validSearchFields == null)
          validSearchFields = propertyNames;

        var sb = new StringBuilder();

        // create dynamic Linq expression
        foreach (String fieldName in validSearchFields)
          sb.AppendFormat("({0} == null ? false : {0}.ToString().IndexOf(@0, @1) >=0) or {1}", fieldName, Environment.NewLine);

        String searchExpression = sb.ToString();
        // remove last "or" occurrence
        searchExpression = searchExpression.Substring(0, searchExpression.LastIndexOf("or"));

        // Apply filtering
        items = items.Where(searchExpression, search, StringComparison.OrdinalIgnoreCase);
      }

      // Skip requires sorting, so make sure there is always sorting
      String sortExpression = "";

      if (propertyNames.Any(c => c == sortBy))
      {
        sortExpression += String.Format("{0} {1}", sortBy, sortDirection);
        items = items.OrderBy(sortExpression);
      }

      if (take == 0)
        take = 100;

      items = items.Skip(skip).Take(take);

      return items;
    }
  }
}
