﻿using AutoMapper;
using BootstrapTable.Wenzhixi;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Models;
using Models.Mappings;
using Security.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

namespace Controllers
{
  public class UserController : Controller
  {
    private readonly UserManager<ApplicationUser> userManager;
    private readonly IMapper mapper;

    public UserController(UserManager<ApplicationUser> userManager)
    {
      this.userManager = userManager;

      // Setup AutoMapper between ApplicationUser and AccountListModel
      var config = new AutoMapper.MapperConfiguration(cfg =>
      {
        cfg.AddProfiles(typeof(AccountMapping).GetTypeInfo().Assembly);
      });

      mapper = config.CreateMapper();
    }

    
    [HttpGet]
    public IActionResult Index()
    {
      return View();
    }


    [HttpGet()]
    [Route("[controller]/[action]")]
    public IActionResult MetaData()
    {
      var tableCreator = new BootStrapTableCreator<AccountListModel>()
      {
        // add edit column
        AllowEdit = true,

        // add delete column
        AllowDelete = true
      };

      // customize phone column
      // find column in collection
      var column = tableCreator.Columns.FirstOrDefault(c => c.field.EqualsEx(nameof(AccountListModel.PhoneNumber)));

      // set value(s)
      column.title = "Phone";

      return tableCreator.Serialize();
    }


    [HttpGet()]
    [Route("[controller]/[action]")]
    public IActionResult Load(String sort, String order, Int32 offset, Int32 limit, String search)
    {
      // apply filtering paging and mapping on datasource
      var tableData = new BootStrapTableData<AccountListModel>(userManager.Users, sort, order, offset, limit, search, mapper);

      // send table data to client
      return tableData.Serialize();
    }
  }
}
