﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.AspNetCore.Identity;
using Security.Models;
using System.Security.Claims;
using System.IO;
using System.Text;
using Newtonsoft.Json;
using System.Reflection;
using Entities;

namespace Microsoft.AspNetCore.Builder
{

  public static class InitDbExtensions
  {
    private enum roles
    {
      Viewer,
      Editor,
      Owner
    }

    public static IApplicationBuilder InitDb(this IApplicationBuilder app)
    {
      var roleManager = app.ApplicationServices.GetService<RoleManager<IdentityRole>>();
      var userManager = app.ApplicationServices.GetService<UserManager<ApplicationUser>>();

      if (userManager.Users.Count() == 0)
      {
        Task.Run(() => InitRoles(roleManager)).Wait();
        Task.Run(() => InitUsers(userManager)).Wait();
      }

      return app;
    }

    private static async Task InitRoles(RoleManager<IdentityRole> roleManager)
    {
      var role = new IdentityRole(roles.Viewer.ToString());
      await roleManager.CreateAsync(role);

      role = new IdentityRole(roles.Editor.ToString());
      await roleManager.CreateAsync(role);
      //await roleManager.AddClaimAsync(role, new Claim("Department", "HR"));

      role = new IdentityRole(roles.Owner.ToString());
      await roleManager.CreateAsync(role);
      //await roleManager.AddClaimAsync(role, new Claim("Department", "HR"));
    }


    private static List<Employee> ReadEmployees()
    {
      // Extract Employee Json file from embedded resource
      var assembly = typeof(InitDbExtensions).GetTypeInfo().Assembly;

      var fileName = assembly.GetManifestResourceNames().FirstOrDefault();

      using (var resourceStream = assembly.GetManifestResourceStream(fileName))
      {
        using (var reader = new StreamReader(resourceStream, Encoding.UTF8))
        {
          var document = reader.ReadToEnd();

          var result = JsonConvert.DeserializeObject<List<Employee>>(document);

          return result;
        }
      }
    }


    private static async Task InitUsers(UserManager<ApplicationUser> userManager)
    {
      foreach (var employee in ReadEmployees())
      {
        var user = new ApplicationUser() { UserName = employee.LastName, Email = employee.Email };
        await userManager.CreateAsync(user, "password");

        if (employee.LastName.EndsWith(".com"))
          await userManager.AddToRoleAsync(user, roles.Viewer.ToString());

        if (employee.LastName.EndsWith(".net"))
          await userManager.AddToRoleAsync(user, roles.Editor.ToString());

        if (employee.LastName.EndsWith(".nl"))
          await userManager.AddToRoleAsync(user, roles.Owner.ToString());
      }
    }
  }
}
